import numpy as np
from enthought.mayavi import mlab
from scipy.integrate import odeint


def V(x, y, z):
    """ A 3D sinusoidal lattice with a parabolic confinement. """
    return np.cos(10*x) + np.cos(10*y) + np.cos(10*z) + 2*(x**2 + y**2 + z**2)

def gradient(f, x, y, z, d=0.01):
    """ Return the gradient of f in (x, y, z). """
    fx  = f(x+d, y, z)
    fx_ = f(x-d, y, z)
    fy  = f(x, y+d, z)
    fy_ = f(x, y-d, z)
    fz  = f(x, y, z+d)
    fz_ = f(x, y, z-d)
    return (fx-fx_)/(2*d), (fy-fy_)/(2*d), (fz-fz_)/(2*d)

def flow(r, t):
    """ The dynamical flow of the system """
    x, y, z, vx, vy, vz = r
    fx, fy, fz = gradient(V, x-.2*np.sin(6*t), y-.2*np.sin(6*t+1), z-.2*np.sin(6*t+2))
    return np.array((vx, vy, vz, -fx - 0.3*vx, -fy - 0.3*vy, -fz - 0.3*vz))


X, Y, Z = np.mgrid[-2:2:100j, -2:2:100j, -2:2:100j]

mlab.contour3d(X, Y, Z, V)
mlab.colorbar(title='Potential', orientation='vertical')

X_new, Y_new, Z_new = np.mgrid[-2:2:5j, -2:2:20j, -2:2:20j]
Vx, Vy, Vz = gradient(V, X_new, Y_new, Z_new)
mlab.quiver3d(X_new, Y_new, Z_new, Vx, Vy, Vz,  scale_factor=-0.01, color=(1, 1, 1))

# Initial conditions
R0 = (0, 0, 0, 0, 0, 0)
# Times at which we want the integrator to return the positions:
t = np.linspace(0, 50, 500)
R = odeint(flow, R0, t)

x, y, z, vx, vy, vz = R.T
trajectory = mlab.plot3d(x, y, z, t, colormap='hot',
                    tube_radius=None)
mlab.show()
